@@ -23,7 +23,7 @@ the second.
 use warnings;
 use strict;
 
-our $VERSION = '0.42';
+our $VERSION = '0.46';
 
 # ------------------------------------------------------------------------------
 # Libraries
@@ -112,18 +112,15 @@ the schema of the first database into that of the second.
 
 sub diff {
     my $self = shift;
-    my $table_re = $self->{opts}{'table-re'};
     my @changes;
+    my %used_tables = ();
 
     debug(1, "\ncomparing databases");
 
     for my $table1 ($self->db1->tables()) {
         my $name = $table1->name();
+        $used_tables{'-- '. $name} = 1;
         debug(4, "table 1 $name = ".Dumper($table1));
-        if ($table_re && $name !~ $table_re) {
-            debug(2,"table '$name' didn't match /$table_re/; ignoring");
-            next;
-        }
         debug(2,"looking at tables called '$name'");
         if (my $table2 = $self->db2->table_by_name($name)) {
             debug(3,"comparing tables called '$name'");
@@ -137,11 +134,8 @@ sub diff {
 
     for my $table2 ($self->db2->tables()) {
         my $name = $table2->name();
+        $used_tables{'-- '. $name} = 1;
         debug(4, "table 2 $name = ".Dumper($table2));
-        if ($table_re && $name !~ $table_re) {
-            debug(2,"table '$name' matched $self->{opts}{'table-re'}; ignoring");
-            next;
-        }
         if (! $self->db1->table_by_name($name)) {
             debug(3,"table '$name' added");
             debug(4,"table '$name' added '".$table2->def()."'");
@@ -154,7 +148,14 @@ sub diff {
 
     my $out = '';
     if (@changes) {
-        $out .= $self->_diff_banner();
+        if (!$self->{opts}{'list-tables'}) {
+            $out .= $self->_diff_banner();
+        }
+        else {
+            $out .= "-- TABLES LIST \n";
+            $out .= join "\n", keys %used_tables;
+            $out .= "\n-- END OF TABLES LIST \n";
+        }
         $out .= join '', @changes;
     }
     return $out;
@@ -193,6 +194,7 @@ sub _diff_tables {
         $self->_diff_fields(@_),
         $self->_diff_indices(@_),
         $self->_diff_primary_key(@_),
+        $self->_diff_foreign_key(@_),
         $self->_diff_options(@_)        
     );
 
@@ -356,6 +358,54 @@ sub _diff_primary_key {
     return @changes;
 }
 
+sub _diff_foreign_key {
+    my ($self, $table1, $table2) = @_;
+
+    my $name1 = $table1->name();
+
+    my $fks1 = $table1->foreign_key();
+    my $fks2 = $table2->foreign_key();
+
+    return () unless $fks1 || $fks2;
+
+    my @changes;
+  
+    if($fks1) {
+        for my $fk (keys %$fks1) {
+            debug(1,"$name1 has fk '$fk'");
+
+            if ($fks2 && $fks2->{$fk}) {
+                if($fks1->{$fk} ne $fks2->{$fk}) 
+                {
+                    debug(1,"foreign key '$fk' changed");
+                    my $changes = "ALTER TABLE $name1 DROP FOREIGN KEY $fk;";
+                    $changes .= " # was CONSTRAINT $fk $fks1->{$fk}"
+                        unless $self->{opts}{'no-old-defs'};
+                    $changes .= "\nALTER TABLE $name1 ADD CONSTRAINT $fk FOREIGN KEY $fks2->{$fk};\n";
+                    push @changes, $changes;
+                }
+            } else {
+                debug(1,"foreign key '$fk' removed");
+                my $changes .= "ALTER TABLE $name1 DROP FOREIGN KEY $fk;";
+                $changes .= " # was CONSTRAINT $fk $fks1->{$fk}"
+                        unless $self->{opts}{'no-old-defs'};
+                $changes .= "\n";
+                push @changes, $changes;
+            }
+        }
+    }
+
+    if($fks2) {
+        for my $fk (keys %$fks2) {
+            next    if($fks1 && $fks1->{$fk});
+            debug(1, "foreign key '$fk' added");
+            push @changes, "ALTER TABLE $name1 ADD CONSTRAINT $fk FOREIGN KEY $fks2->{$fk};\n";
+        }
+    }
+
+    return @changes;
+}
+
 # If we're about to drop a composite (multi-column) index, we need to
 # check whether any of the columns in the composite index are
 # auto_increment; if so, we have to add an index for that
@@ -424,7 +474,7 @@ sub _load_database {
     }
 
     if ($arg =~ /^db:(.*)/) {
-        return MySQL::Diff::Database->new(db => $1, auth => \%auth);
+        return MySQL::Diff::Database->new(db => $1, auth => \%auth, 'table-re' => $self->{opts}{'table-re'});
     }
 
     if ($self->{opts}{"dbh"}              ||
@@ -433,18 +483,18 @@ sub _load_database {
         $self->{opts}{"user$authnum"}     ||
         $self->{opts}{"password$authnum"} ||
         $self->{opts}{"socket$authnum"}) {
-        return MySQL::Diff::Database->new(db => $arg, auth => \%auth);
+        return MySQL::Diff::Database->new(db => $arg, auth => \%auth, 'table-re' => $self->{opts}{'table-re'});
     }
 
     if (-f $arg) {
-        return MySQL::Diff::Database->new(file => $arg, auth => \%auth);
+        return MySQL::Diff::Database->new(file => $arg, auth => \%auth, 'table-re' => $self->{opts}{'table-re'});
     }
 
     my %dbs = MySQL::Diff::Database::available_dbs(%auth);
     debug(2, "  available databases: ", (join ', ', keys %dbs), "\n");
 
     if ($dbs{$arg}) {
-        return MySQL::Diff::Database->new(db => $arg, auth => \%auth);
+        return MySQL::Diff::Database->new(db => $arg, auth => \%auth, 'table-re' => $self->{opts}{'table-re'});
     }
 
     warn "'$arg' is not a valid file or database.\n";
@@ -462,7 +512,7 @@ __END__
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2000-2011 Adam Spiers. All rights reserved. This
+Copyright (c) 2000-2016 Adam Spiers. All rights reserved. This
 program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
